@@ -2,6 +2,7 @@
 
 ;; Copyright (C) 1985, 86, 87, 93, 94, 1995, 1996 Free Software Foundation, Inc.
 ;; Copyright (C) 1995 Tinker Systems and INS Engineering Corp.
+;; Copyright (C) 2001 Ben Wing.
 
 ;; Author: Roland McGrath <roland@prep.ai.mit.edu>
 ;; Maintainer: XEmacs Development Team
@@ -317,6 +318,12 @@ of[ \t]+\"?\\([a-zA-Z]?:?[^\":\n]+\\)\"?:" 3 2)
    (ada
     ("\\([^, \n\t]+\\), line \\([0-9]+\\), char \\([0-9]+\\)[:., \(-]" 1 2 3)
     )
+
+    ;; Ant / cygwin:
+    ;; file:G:/foobar/dev/build-myproj.xml:54: Compiler Adapter 'javac' can't be found.
+   (ant
+    ("file:\\(\\([a-zA-Z]:\\)?[^:(\t\n]+\\):[ \t]*\\([0-9]+\\)[: \t]" 1 3)
+    )
    )
   "Alist of (system regexp-alist) for building
 `compilation-error-regexp-alist'.  SYSTEM is either a system identifier,
@@ -378,6 +385,17 @@ See also the variable `compilation-error-regexp-alist-alist'."
 
 (compilation-build-compilation-error-regexp-alist)
 
+(defcustom compilation-error-screen-columns t
+  "*If non-nil, column numbers in error messages are screen columns.
+Otherwise they are interpreted as character positions, with
+each character occupying one column.
+The default is to use screen columns, which requires that the compilation
+program and Emacs agree about the display width of the characters,
+especially the TAB character."
+:type 'boolean
+:group 'compilation
+:version "20.4")
+
 (defcustom compilation-read-command t
   "*If not nil, M-x compile reads the compilation command to use.
 Otherwise, M-x compile just uses the value of `compile-command'."
@@ -425,6 +443,24 @@ by influencing the default value for the variable `grep-find-command'.")
 		 (+ 22 (length grep-command)))))
   "The default find command for \\[grep-find].")
 
+;; XEmacs addition: all grep-all-files stuff
+
+(defvar grep-all-files-history nil)
+
+(defcustom grep-all-files-omitted-expressions
+  '("*~" "#*" ".#*" ",*" "*.elc" "*.obj" "*.o" "*.exe" "*.dll" "*.lib" "*.a"
+    "*.dvi" "*.class" "*.bin" "*.orig" "*.rej")
+  "List of expressions matching files to be omitted in `grep-all-files-...'.
+Each entry should be a simple name or a shell wildcard expression."
+  :type '(repeat string)
+  :group 'compilation)
+
+(defcustom grep-all-files-omitted-directories '("CVS" "RCS" "SCCS")
+  "List of directories not to recurse into in `grep-all-files-...'.
+Each entry should be a simple name or a shell wildcard expression."
+  :type '(repeat string)
+  :group 'compilation)
+
 ;;;###autoload
 (defcustom compilation-search-path '(nil)
   "*List of directories to search for source files named in error messages.
@@ -607,6 +643,59 @@ easily repeat a find command."
   (let ((grep-null-device nil))		; see grep
     (grep command-args)))
 
+;; XEmacs addition: all grep-all-files stuff
+
+(defun construct-grep-all-files-command (find-segment grep-segment)
+  (let ((omit-annoying
+	 (mapconcat #'(lambda (wildcard)
+			(concat "-name '" wildcard "' -or "))
+		    grep-all-files-omitted-expressions
+		    "")))
+    (cond ((eq grep-find-use-xargs 'gnu)
+	   (format "find . %s %s -type f -print0 | xargs -0 -e %s"
+		   find-segment omit-annoying grep-segment))
+	  (grep-find-use-xargs
+	   (format "find . %s %s -type f -print | xargs %s"
+		   find-segment omit-annoying grep-segment))
+	  (t
+	   (format "find . %s %s -type f -exec %s {} /dev/null \\;"
+		   find-segment omit-annoying grep-segment)))))
+
+;;;###autoload
+(defun grep-all-files-in-current-directory (command)
+  "Run `grep' in all non-annoying files in the current directory.
+`Non-annoying' excludes backup files, autosave files, CVS merge files, etc.
+More specifically, this is controlled by `grep-all-files-omitted-expressions'.
+
+This function does not recurse into subdirectories.  If you want this,
+use \\[grep-all-files-in-current-directory-and-below]."
+  (interactive
+   (progn
+     (list (read-shell-command "Run grep (like this): "
+			       grep-command 'grep-all-files-history))))
+  (grep (construct-grep-all-files-command
+	 "-name . -or -type d -prune -or" command)))
+
+;;;###autoload
+(defun grep-all-files-in-current-directory-and-below (command)
+  "Run `grep' in all non-annoying files in the current directory and below.
+`Non-annoying' excludes backup files, autosave files, CVS merge files, etc.
+More specifically, this is controlled by `grep-all-files-omitted-expressions'.
+
+This function recurses into subdirectories.  If you do not want this,
+use \\[grep-all-files-in-current-directory]."
+  (interactive
+   (progn
+     (list (read-shell-command "Run grep (like this): "
+			       grep-command 'grep-all-files-history))))
+  (grep (construct-grep-all-files-command
+	 ;; prune all specified directories.
+	 (mapconcat #'(lambda (wildcard)
+			(concat "-name '" wildcard "' -prune -or "))
+		    grep-all-files-omitted-directories
+		    "")
+	 command)))
+
 (defun compile-internal (command error-message
 				 &optional name-of-mode parser regexp-alist
 				 name-function)
@@ -683,6 +772,7 @@ Returns the compilation buffer created."
 	(save-window-excursion
 	  (select-window outwin)
 	  (goto-char (point-max)))
+	(setq outwin (display-buffer outbuf))
 	
 	;; XEmacs change
 	(compilation-mode name-of-mode)
@@ -945,13 +1035,13 @@ Runs `compilation-mode-hook' with `run-hooks' (which see)."
   (set (make-local-variable 'compilation-old-error-list) nil)
   (set (make-local-variable 'compilation-parsing-end) 1)
   (set (make-local-variable 'compilation-directory-stack) nil)
+  (make-local-variable 'compilation-error-screen-columns)
   (setq compilation-last-buffer (current-buffer))
   ;; XEmacs change: highlight lines, install menubar.
   (require 'mode-motion)
   (setq mode-motion-hook 'compilation-mode-motion-hook)
-  (make-local-variable 'mouse-track-click-hook)
-  (add-hook 'mouse-track-click-hook 'compile-mouse-maybe-goto-error)
-  )
+  (make-local-hook 'mouse-track-click-hook)
+  (add-hook 'mouse-track-click-hook 'compile-mouse-maybe-goto-error t t))
 
 ;;;###autoload
 (defvar compilation-shell-minor-mode nil
@@ -1551,10 +1641,12 @@ The current buffer should be the desired compilation output buffer."
 			    ;; Look for the next error.
 			    t)
 			;; We found the file.  Get a marker for this error.
-			;; compilation-old-error-list is a buffer-local
-			;; variable, so we must be careful to extract its value
+			;; compilation-old-error-list and
+			;; compilation-error-screen-columns are buffer-local
+			;; so we must be careful to extract their value
 			;; before switching to the source file buffer.
 			(let ((errors compilation-old-error-list)
+			      (columns compilation-error-screen-columns)
 			      (last-line (nth 1 (cdr next-error)))
 			      (column (nth 2 (cdr next-error))))
 			  (set-buffer buffer)
@@ -1564,7 +1656,9 @@ The current buffer should be the desired compilation output buffer."
 			      (goto-line last-line)
 			      (if (and column (> column 0))
 				  ;; Columns in error msgs are 1-origin.
-				  (move-to-column (1- column))
+				  (if columns
+				      (move-to-column (1- column))
+				    (forward-char (1- column)))
 				(beginning-of-line))
 			      (setcdr next-error (point-marker))
 			      ;; Make all the other error messages referring
@@ -1588,7 +1682,9 @@ The current buffer should be the desired compilation output buffer."
                                                                 lines))
                                          (forward-line lines))
 				       (if (and column (> column 1))
-					   (move-to-column (1- column))
+					   (if columns
+					       (move-to-column (1- column))
+					     (forward-char (1- column)))
 					 (beginning-of-line))
 				       (setq last-line this)
 				       (setcdr (car errors) (point-marker))))
@@ -1710,7 +1806,7 @@ bTo BUFFER : ")
     (let* ((dirs compilation-search-path)
 	   (true-file-name (file-name-nondirectory filename))
 	   (buffer (get-buffer true-file-name)) 
-	   fi thisdir fmts name)
+	   thisdir fmts name)
       (flet ((push-file-name-history (name)
 	       (setq file-name-history
 		     (if minibuffer-history-uniquify
@@ -2074,6 +2170,12 @@ See variable `compilation-parse-errors-function' for the interface it uses."
   (setq compilation-error-list (nreverse compilation-error-list))
   (display-message 'progress "Parsing error messages...done"))
 
+(defun compile-buffer-substring (index)
+  "Get substring matched by INDEXth subexpression."
+  (if index
+      (let ((beg (match-beginning index)))
+	(if beg (buffer-substring beg (match-end index))))))
+
 ;; If directory DIR is a subdir of ORIG or of ORIG's parent,
 ;; return a relative name for it starting from ORIG or its parent.
 ;; ORIG-EXPANDED is an expanded version of ORIG.
